@@ -17,7 +17,7 @@ from feeluown.library import ProviderFlags, ModelState, NotSupported, ModelFlags
 from feeluown.models import reverse, ModelType
 
 from feeluown.gui.helpers import async_run, BgTransparentMixin, disconnect_slots_if_has
-from feeluown.widgets import TextButton
+from feeluown.widgets.imglist import ImgListView
 from feeluown.widgets.album import AlbumListModel, AlbumListView, AlbumFilterProxyModel
 from feeluown.widgets.artist import ArtistListModel, ArtistListView, \
     ArtistFilterProxyModel
@@ -139,11 +139,10 @@ class Renderer:
 
     def _show_model_with_cover(self, reader, table, model_cls, filter_model_cls):
         self.container.current_table = table
-        filter_model = filter_model_cls(self.albums_table)
+        filter_model = filter_model_cls()
         source_name_map = {p.identifier: p.name for p in self._app.library.list()}
         model = model_cls(reader,
                           fetch_cover_wrapper(self._app.img_mgr),
-                          parent=self.artists_table,
                           source_name_map=source_name_map)
         filter_model.setSourceModel(model)
         table.setModel(filter_model)
@@ -362,35 +361,6 @@ class VideosRenderer(Renderer):
         self.show_videos(self.reader)
 
 
-class PlayerPlaylistRenderer(Renderer):
-
-    async def render(self):
-        self.meta_widget.title = '当前播放列表'
-        self.meta_widget.show()
-        player = self._app.player
-        playlist = player.playlist
-
-        async def clear_playlist():
-            playlist.clear()
-            await self.render()  # re-render
-
-        songs = playlist.list()
-        self.show_songs(wrap(songs.copy()))
-        btn = TextButton('清空', self.toolbar)
-        btn.clicked.connect(lambda *args: aio.create_task(clear_playlist()))
-        self.toolbar.add_tmp_button(btn)
-
-        self.songs_table.remove_song_func = playlist.remove
-
-        # scroll to current song
-        current_song = self._app.playlist.current_song
-        if current_song is not None:
-            row = songs.index(current_song)
-            model_index = self.songs_table.model().index(row, 0)
-            self.songs_table.scrollTo(model_index)
-            self.songs_table.selectRow(row)
-
-
 class DescLabel(QLabel):
     def __init__(self, parent=None):
         super().__init__(parent=parent)
@@ -510,6 +480,8 @@ class TableContainer(QFrame, BgTransparentMixin):
                 self.toolbar.albums_mode()
             if table is self.songs_table:
                 self.toolbar.songs_mode()
+        if isinstance(self._table, ImgListView):
+            self._table.setModel(None)
         self._table = table
 
     async def set_renderer(self, renderer):
@@ -620,9 +592,6 @@ class TableContainer(QFrame, BgTransparentMixin):
     def show_artists_coll(self, artists_g):
         aio.create_task(self.set_renderer(ArtistsCollectionRenderer(artists_g)))
 
-    def show_player_playlist(self):
-        aio.create_task(self.set_renderer(PlayerPlaylistRenderer()))
-
     def search(self, text):
         if self.isVisible() and self.songs_table is not None:
             self.songs_table.filter_row(text)
