@@ -36,13 +36,13 @@ class Ui:
         self.sidebar = self._left_panel_con = LeftPanel(self._app,)
         self.left_panel = self._left_panel_con.p
         self.right_panel = RightPanel(self._app, self._splitter)
-        self.bottom_panel = self.right_panel.bottom_panel
+        self.toolbar = self.bottom_panel = self.right_panel.bottom_panel
         self.mpv_widget = MpvOpenGLWidget(self._app)
         self.frameless_container = None
 
         # alias
         self.magicbox = self.bottom_panel.magicbox
-        self.pc_panel = self.top_panel.pc_panel
+        self.player_bar = self.pc_panel = self.top_panel.pc_panel
         self.table_container = self.right_panel.table_container
         # backward compatible, old name is songs_table_container
         self.songs_table_container = self.table_container
