@@ -1,20 +1,19 @@
 import asyncio
 import logging
 
-from PyQt5.QtCore import Qt, QTimer, QRect
+from PyQt5.QtCore import Qt, QTimer, QRect, QEvent
 from PyQt5.QtGui import QFontMetrics, QPainter
 from PyQt5.QtWidgets import (
     QApplication, QLabel, QFrame, QHBoxLayout, QVBoxLayout,
     QPushButton, QSizePolicy, QMenu,
 )
-
 from feeluown.library import ProviderFlags
 from feeluown.utils import aio
 from feeluown.excs import ProviderIOError
 from feeluown.media import MediaType
 from feeluown.player import PlaybackMode, State
 from feeluown.gui.widgets.lyric import Window as LyricWindow
-from feeluown.gui.helpers import async_run
+from feeluown.gui.helpers import async_run, resize_font
 from feeluown.widgets import TextButton
 from feeluown.widgets.volume_button import VolumeButton
 from feeluown.widgets.progress_slider import ProgressSlider
@@ -23,6 +22,92 @@ from feeluown.gui.widgets.labels import ProgressLabel, DurationLabel
 logger = logging.getLogger(__name__)
 
 
+class LikeButton(QPushButton):
+    def __init__(self, app, parent=None):
+        super().__init__(parent=parent)
+        self._app = app
+        self.setCheckable(True)
+        self._app.playlist.song_changed_v2.connect(self.on_song_changed)
+        self.clicked.connect(self.toggle_liked)
+        self.toggled.connect(self.on_toggled)
+
+    def on_song_changed(self, song, media):
+        if song is not None:
+            self.setChecked(self.is_song_liked(song))
+
+    def toggle_liked(self):
+        song = self._app.playlist.current_song
+        coll_library = self._app.coll_uimgr.get_coll_library()
+        if self.is_song_liked(song):
+            coll_library.remove(song)
+            self._app.show_msg('歌曲已经从“本地收藏”中移除')
+        else:
+            coll_library.add(song)
+            self._app.show_msg('歌曲已经添加到“本地收藏”')
+
+    def on_toggled(self):
+        song = self._app.playlist.current_song
+        if self.is_song_liked(song):
+            self.setToolTip('添加到“本地收藏”')
+        else:
+            self.setToolTip('从“本地收藏”中移除')
+
+    def is_song_liked(self, song):
+        coll_library = self._app.coll_uimgr.get_coll_library()
+        return song in coll_library.models
+
+
+class WatchButton(TextButton):
+    def __init__(self, app, parent):
+        super().__init__('♨', parent=parent)
+        self._app = app
+
+        self.setToolTip('开启 watch 模式时，播放器会优先尝试为歌曲找一个合适的视频来播放。\n'
+                        '最佳实践：开启 watch 的同时建议开启视频的画中画模式。')
+
+        self.setCheckable(True)
+        self._setup_ui()
+        self.clicked.connect(self.on_clicked)
+
+    def on_clicked(self):
+        self._app.playlist.watch_mode = not self._app.playlist.watch_mode
+        # Assume that playlist.watch_mode will only be changed by WatchButton
+        if self._app.playlist.watch_mode is True:
+            self.setChecked(True)
+        else:
+            self.setChecked(False)
+
+    def _setup_ui(self):
+        font = self.font()
+        resize_font(font, -2)
+        self.setFont(font)
+
+
+class LyricButton(TextButton):
+    def __init__(self, app, parent):
+        super().__init__('词', parent=parent)
+        self._app = app
+
+        self.setCheckable(True)
+        self.clicked.connect(self._toggle_lyric_window)
+        self.parent().lyric_window.installEventFilter(self)  # hack
+
+    def _toggle_lyric_window(self):
+        lyric_window = self._app.ui.player_bar.lyric_window
+        if lyric_window.isVisible():
+            lyric_window.hide()
+        else:
+            lyric_window.show()
+
+    def eventFilter(self, obj, event):
+        """Event filter for lyric window"""
+        if event.type() == QEvent.Show:
+            self.setChecked(True)
+        elif event.type() == QEvent.Hide:
+            self.setChecked(False)
+        return False
+
+
 class SongBriefLabel(QLabel):
     default_text = '...'
 
@@ -156,6 +241,45 @@ class SongBriefLabel(QLabel):
                 task.add_done_callback(artists_fetched_cb)
 
 
+class SourceLabel(QLabel):
+    default_text = '音乐来源'
+
+    def __init__(self, app, parent=None):
+        super().__init__(text=SourceLabel.default_text, parent=parent)
+        self._app = app
+
+    def contextMenuEvent(self, e):
+        # FIXME(wuliaotc): 在切换provider时禁用menu
+        song = self._app.playlist.current_song
+        if song is None:
+            return
+
+        menu = QMenu()
+        submenu = menu.addMenu('“智能”替换')
+        for provider in self._app.library.list():
+            pid = provider.identifier
+            if pid == song.source:
+                continue
+            action = submenu.addAction(provider.name)
+            action.triggered.connect(
+                (lambda x: lambda: asyncio.create_task(self._switch_provider(x)))(pid)
+            )
+        menu.exec(e.globalPos())
+
+    async def _switch_provider(self, provider_id):
+        song = self._app.playlist.current_song
+        songs = await self._app.library.a_list_song_standby(
+            song, source_in=[provider_id])
+        if songs:
+            standby = songs[0]
+            assert standby != song
+            self._app.show_msg(f'使用 {standby} 替换当前歌曲')
+            self._app.playlist.pure_set_current_song(standby, standby.url)
+            self._app.playlist.remove(song)
+        else:
+            self._app.show_msg(f'提供方 “{provider_id}” 没有找到可用的相似歌曲')
+
+
 class PlayerControlPanel(QFrame):
 
     def __init__(self, app, parent=None):
@@ -173,6 +297,8 @@ class PlayerControlPanel(QFrame):
             PlaybackMode.loop: '循环播放',
             PlaybackMode.random: '随机播放',
         }
+        self.lyric_window = LyricWindow()
+        self.lyric_window.hide()
 
         # initialize sub widgets
         self._layout = QHBoxLayout(self)
@@ -185,17 +311,15 @@ class PlayerControlPanel(QFrame):
         self.volume_btn = VolumeButton(self)
         self.playlist_btn = IconButton(parent=self)
         #: mark song as favorite button
-        self.like_btn = QPushButton(self)
+        self.like_btn = LikeButton(self._app, self)
         self.mv_btn = TextButton('MV', self)
-        self.lyric_btn = TextButton('词', self)
+        self.toggle_lyric_btn = LyricButton(self._app, self)
         self.download_btn = QPushButton(self)
+        self.toggle_watch_btn = WatchButton(self._app, self)
         self.toggle_video_btn = TextButton('△', self)
         # toggle picture-in-picture button
         self.toggle_pip_btn = TextButton('◲', self)
 
-        self.lyric_window = LyricWindow()
-        self.lyric_window.hide()
-
         self.previous_btn.setObjectName('previous_btn')
         self.pp_btn.setObjectName('pp_btn')
         self.next_btn.setObjectName('next_btn')
@@ -205,7 +329,7 @@ class PlayerControlPanel(QFrame):
         self.download_btn.setObjectName('download_btn')
         self.like_btn.setObjectName('like_btn')
         self.mv_btn.setObjectName('mv_btn')
-        self.lyric_btn.setObjectName('lyric_btn')
+        self.toggle_lyric_btn.setObjectName('toggle_lyric_btn')
         self.toggle_video_btn.setObjectName('toggle_video_btn')
         self.toggle_pip_btn.setObjectName('toggle_pip_btn')
 
@@ -217,16 +341,15 @@ class PlayerControlPanel(QFrame):
 
         self.mv_btn.setToolTip('播放 MV')
         self.download_btn.setToolTip('下载歌曲（未实现，欢迎 PR）')
-        self.like_btn.setToolTip('收藏歌曲（未实现，欢迎 PR）')
         self.pp_btn.setCheckable(True)
-        self.like_btn.setCheckable(True)
         self.download_btn.setCheckable(True)
         self.toggle_video_btn.hide()
         self.toggle_pip_btn.hide()
 
         self.song_title_label = SongBriefLabel(self._app)
-        self.song_source_label = QLabel('歌曲来源', parent=self)
         self.song_title_label.setAlignment(Qt.AlignCenter)
+        self.song_source_label = SourceLabel(self._app, parent=self)
+
         self.duration_label = DurationLabel(app, parent=self)
         self.position_label = ProgressLabel(app, parent=self)
 
@@ -238,13 +361,13 @@ class PlayerControlPanel(QFrame):
         self.previous_btn.clicked.connect(self._app.playlist.previous)
         self.pp_btn.clicked.connect(self._app.player.toggle)
         self.pms_btn.clicked.connect(self._switch_playback_mode)
-        self.lyric_btn.clicked.connect(self._toggle_lyric_window)
         self.volume_btn.change_volume_needed.connect(
             lambda volume: setattr(self._app.player, 'volume', volume))
 
         player = self._app.player
 
-        player.state_changed.connect(self._on_player_state_changed, aioqueue=True)
+        player.state_changed.connect(self._on_player_state_changed,
+                                     aioqueue=True)
         player.playlist.playback_mode_changed.connect(
             self.on_playback_mode_changed, aioqueue=True)
         player.playlist.song_changed.connect(
@@ -252,7 +375,8 @@ class PlayerControlPanel(QFrame):
         player.media_changed.connect(
             self.on_player_media_changed, aioqueue=True)
         player.volume_changed.connect(self.volume_btn.on_volume_changed)
-        self._app.live_lyric.sentence_changed.connect(self.lyric_window.set_sentence)
+        self._app.live_lyric.sentence_changed.connect(
+            self.lyric_window.set_sentence)
         self.lyric_window.play_previous_needed.connect(player.playlist.previous)
         self.lyric_window.play_next_needed.connect(player.playlist.next)
 
@@ -269,10 +393,13 @@ class PlayerControlPanel(QFrame):
         self.position_label.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
         self.like_btn.setFixedSize(15, 15)
         self.download_btn.setFixedSize(15, 15)
+        self.download_btn.hide()
         self.mv_btn.setFixedHeight(16)
-        self.lyric_btn.setFixedHeight(16)
+        self.toggle_lyric_btn.setFixedHeight(16)
+        self.toggle_watch_btn.setFixedHeight(16)
 
-        self.progress_slider.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Preferred)
+        self.progress_slider.setSizePolicy(QSizePolicy.Expanding,
+                                           QSizePolicy.Preferred)
         self._sub_layout = QVBoxLayout()
         self._sub_top_layout = QHBoxLayout()
 
@@ -286,9 +413,11 @@ class PlayerControlPanel(QFrame):
         self._sub_top_layout.addSpacing(8)
         self._sub_top_layout.addWidget(self.mv_btn)
         self._sub_top_layout.addSpacing(8)
-        self._sub_top_layout.addWidget(self.lyric_btn)
+        self._sub_top_layout.addWidget(self.toggle_lyric_btn)
         self._sub_top_layout.addSpacing(8)
-        self._sub_top_layout.addWidget(self.download_btn)
+        self._sub_top_layout.addWidget(self.toggle_watch_btn)
+        # self._sub_top_layout.addSpacing(8)
+        # self._sub_top_layout.addWidget(self.download_btn)
         self._sub_top_layout.addSpacing(3)
 
         self._sub_layout.addSpacing(3)
@@ -356,8 +485,8 @@ class PlayerControlPanel(QFrame):
         source_name = source_name_map.get(song.source, song.source)
         self.song_source_label.setText(source_name)
         self.song_title_label.setText(text)
-        loop = asyncio.get_event_loop()
-        loop.create_task(self.update_mv_btn_status(song))
+        task_spec = self._app.task_mgr.get_or_create('update-mv-btn-status')
+        task_spec.bind_coro(self.update_mv_btn_status(song))
 
     def on_player_media_changed(self, media):
         if media is not None and media.type_ == MediaType.audio:
@@ -385,12 +514,6 @@ class PlayerControlPanel(QFrame):
     def _on_player_state_changed(self, state):
         self.pp_btn.setChecked(state == State.playing)
 
-    def _toggle_lyric_window(self):
-        if self.lyric_window.isVisible():
-            self.lyric_window.hide()
-        else:
-            self.lyric_window.show()
-
 
 class TopPanel(QFrame):
     def __init__(self, app, parent=None):
