@@ -206,14 +206,20 @@ class Library:
                     break
         return result
 
-    async def a_list_song_standby(self, song, onlyone=True):
+    async def a_list_song_standby(self, song, onlyone=True, source_in=None):
         """async version of list_song_standby
+
+        .. versionadded:: 3.7.5
+             The *source_in* paramter.
         """
-        providers = self._providers_standby or [pvd.identifier for pvd in self.list()]
+        if source_in is None:
+            pvd_ids = self._providers_standby or [pvd.identifier for pvd in self.list()]
+        else:
+            pvd_ids = [pvd.identifier for pvd in self._filter(identifier_in=source_in)]
         # FIXME(cosven): the model return from netease is new model,
         # and it does not has url attribute
-        valid_providers = [provider for provider in providers
-                           if provider != song.source and provider != 'netease']
+        valid_providers = [pvd_id for pvd_id in pvd_ids
+                           if pvd_id != song.source and pvd_id != 'netease']
         q = '{} {}'.format(song.title_display, song.artists_name_display)
         result_g = []
         async for result in self.a_search(q, source_in=valid_providers):
@@ -233,6 +239,7 @@ class Library:
                     if onlyone or len(result) >= 2:
                         break
         return result
+
     #
     # methods for v2
     #
@@ -330,6 +337,30 @@ class Library:
                     media = Media(url)
                 else:
                     raise MediaNotFound
+        if not media:
+            raise MediaNotFound
+        return media
+
+    def song_prepare_mv_media(self, song: BriefSongProtocol, policy) -> Media:
+        """
+
+        .. versionadded:: 3.7.5
+        """
+        provider = self.get(song.source)
+        if provider is None:
+            raise MediaNotFound(f'provider:{song.source} not found')
+        song_v1 = self.cast_model_to_v1(song)
+        mv = song_v1.mv
+        if mv.meta.support_multi_quality:
+            media, _ = mv.select_media(policy)
+        else:
+            media = mv.media
+            if media:
+                media = Media(media)
+            else:
+                media = None
+        if not media:
+            raise MediaNotFound
         return media
 
     def song_get_lyric(self, song: BriefSongModel):
